use specs::World;

use crate::map_builders::bsp_dungeon::BspDungeonBuilder;
use crate::map_builders::simple_map::SimpleMapBuilder;
use crate::{Map, Position};

mod bsp_dungeon;
mod common;
mod simple_map;

pub trait MapBuilder {
    fn build_map(&mut self);
    fn spawn_entities(&mut self, ecs: &mut World);
    fn get_map(&self) -> Map;
    fn get_starting_position(&self) -> Position;
    fn get_snapshot_history(&self) -> Vec<Map>;
    fn take_snapshot(&mut self);
}

pub fn new_random_builder(new_depth: i32) -> Box<dyn MapBuilder> {
    let mut rng = rltk::RandomNumberGenerator::new();
    let builder_choice = rng.roll_dice(1, 2);
    match builder_choice {
        1 => Box::new(SimpleMapBuilder::new(new_depth)),
        _ => Box::new(BspDungeonBuilder::new(new_depth)),
    }
}
